/*assume:
10000 patients
binary outcome
binary treatment 50-50
continuous covariate confounded with treatment
*/
program getsim
	cd "P:\Evan\Methodological\Outcome MI\"
	//cd "D:\Dropbox\work\temp\Outcome MI\"
	set more off
	set maxiter 30
	tempfile tempf
	set seed 7
	set matsize 10000
	//inputs
	local N=10000
	local cases=100000

	di in green _newline(2) "calc true effects:"
	qui clear
	//matrices
	matrix mat1 = J(`N',4,.)
	matrix colnames mat1 = ORtre ORage ORint corr
	matrix mat2 = J(`N',3,.)
	matrix colnames mat2 = up_ORtre up_ORage up_ORint
	matrix mat3 = J(`N',3,.)
	matrix colnames mat3 = lo_ORtre lo_ORage lo_ORint
	matrix mat4 = J(`N',1,.)
	matrix colnames mat4 = ROC
	matrix mat5 = J(`N',6,.)
	matrix colnames mat5 = OR2tre OR2age OR2int tcorr Pr2A Pr2B 	
	
	forvalues i=1(1)`N' {
		qui {
			clear
			matrix C = (1, .5 \ .5, 1)
			drawnorm x age, n(`cases') corr(C)
			//no need to edit "age" to make its mean and sd recognisable. it affects the associations with the outcome and does not add much to the hypothesis
			//generate treatment
			gen treatment=0
			replace treatment=1 if x>0
			//generate outcome Pr(Y=1|group=0)=0.1 / Pr(Y=1|group=1)=0.18182 (for OR=2)
			gen outcome = uniform() < invlogit(-2.19722 + ln(2)*treatment + ln(1.5)*age + ln(1.2)*treatment*age)
			logit outcome treatment age i.treatment#c.age, or
			//calculate some metrics for the data generation process
			matrix mat1[`i',1]=exp(_b[treatment])
			matrix mat1[`i',2]=exp(_b[age])
			matrix mat1[`i',3]=exp(_b[1.treatment#c.age])
			corr age treatment
			matrix mat1[`i',4]=r(rho)
			matrix mat2[`i',1]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
			matrix mat2[`i',2]=exp(_b[age]+invnormal(0.975)*_se[age])
			matrix mat2[`i',3]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
			matrix mat3[`i',1]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
			matrix mat3[`i',2]=exp(_b[age]-invnormal(0.975)*_se[age])
			matrix mat3[`i',3]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])
			//AUC
			lroc, nograph
			matrix mat4[`i',1]=r(area)
			//generate second outcome
			gen outcome2 = uniform() < invlogit(0 + ln(5)*outcome)
			logit outcome2 treatment age i.treatment#c.age, or
			matrix mat5[`i',1]=exp(_b[treatment])
			matrix mat5[`i',2]=exp(_b[age])
			matrix mat5[`i',3]=exp(_b[1.treatment#c.age])
			tetrachoric outcome outcome2
			matrix mat5[`i',4]=r(rho)
			count if treatment==0
			local t1=r(N)
			count if treatment==0 & outcome2==1
			matrix mat5[`i',5]=r(N)/`t1' /*P(Y=1|E=0)*/
			count if treatment==1
			local t2=r(N)
			count if treatment==1 & outcome2==1
			matrix mat5[`i',6]=r(N)/`t2' /*P(Y=1|E=1)*/
		}
		if mod(`i',10)==0 {
			di in yellow "." _continue
		}
		if mod(`i',500)==0 {
			di in yellow "`i'"
		}
	}
	clear
	qui svmat mat1, names(col)
	qui svmat mat2, names(col)
	qui svmat mat3, names(col)
	qui svmat mat4, names(col)
	qui svmat mat5, names(col)
	qui compress
	save true_`N'_`cases'.dta ,replace
end
