/*assume:
10000 patients
binary outcome
binary treatment 50-50
continuous covariate confounded with treatment
*/
capture program drop getsim
program getsim
	//cd "P:\Evan\Methodological\Outcome MI\"
	//cd "D:\Dropbox\work\temp\Outcome MI\"
	//cd "/mnt/iusers01/ja01/mdmysek3/Stata/OutcomeMI/"
	set more off
	set maxiter 30
	tempfile tempx tempf temph
	set seed 7
	set matsize 10000
	//inputs
	local N=`1'		/*number of replications*/
	local cases=`2'	/*number of cases in each dataset*/
	local msizelim=10
	//decide on matrix size
	local msz=`N'
	if `N'>`msizelim' {
		local msz=`msizelim'
	}	
	
	//allow roc with argument
	local txt="noROC"
	if "`1'"=="roc" {
		local txt="ALL"
	}

	//missing rate loop
	forvalues mrate=0.8(0.2)0.8 {
		di in green _newline(2) "`txt' cases=`cases' missrate=`=string(round(`mrate',0.1))':"
		//local mrate=0.2
		qui clear
		//matrices
		matrix mat1 = J(`msz',49,.)
		matrix colnames mat1 = ORtre ORage ORint corr /*
		*/ mcar1ORtre mcar1ORage mcar1ORint mcar2ORtre mcar2ORage mcar2ORint mcar3ORtre mcar3ORage mcar3ORint /*
		*/ mcar4ORtre mcar4ORage mcar4ORint mcar5ORtre mcar5ORage mcar5ORint /*
		*/ mar1ORtre mar1ORage mar1ORint mar2ORtre mar2ORage mar2ORint mar3ORtre mar3ORage mar3ORint /*
		*/ mar4ORtre mar4ORage mar4ORint mar5ORtre mar5ORage mar5ORint /*
		*/ mnar1ORtre mnar1ORage mnar1ORint mnar2ORtre mnar2ORage mnar2ORint mnar3ORtre mnar3ORage mnar3ORint /*
		*/ mnar4ORtre mnar4ORage mnar4ORint mnar5ORtre mnar5ORage mnar5ORint
		matrix mat2 = J(`msz',48,.)
		matrix colnames mat2 = up_ORtre up_ORage up_ORint /*
		*/ up_mcar1ORtre up_mcar1ORage up_mcar1ORint up_mcar2ORtre up_mcar2ORage up_mcar2ORint up_mcar3ORtre up_mcar3ORage up_mcar3ORint /*
		*/ up_mcar4ORtre up_mcar4ORage up_mcar4ORint up_mcar5ORtre up_mcar5ORage up_mcar5ORint /*
		*/ up_mar1ORtre up_mar1ORage up_mar1ORint up_mar2ORtre up_mar2ORage up_mar2ORint up_mar3ORtre up_mar3ORage up_mar3ORint /*
		*/ up_mar4ORtre up_mar4ORage up_mar4ORint up_mar5ORtre up_mar5ORage up_mar5ORint /*
		*/ up_mnar1ORtre up_mnar1ORage up_mnar1ORint up_mnar2ORtre up_mnar2ORage up_mnar2ORint up_mnar3ORtre up_mnar3ORage up_mnar3ORint /*
		*/ up_mnar4ORtre up_mnar4ORage up_mnar4ORint up_mnar5ORtre up_mnar5ORage up_mnar5ORint
		matrix mat3 = J(`msz',48,.)
		matrix colnames mat3 = lo_ORtre lo_ORage lo_ORint /*
		*/ lo_mcar1ORtre lo_mcar1ORage lo_mcar1ORint lo_mcar2ORtre lo_mcar2ORage lo_mcar2ORint lo_mcar3ORtre lo_mcar3ORage lo_mcar3ORint /*
		*/ lo_mcar4ORtre lo_mcar4ORage lo_mcar4ORint lo_mcar5ORtre lo_mcar5ORage lo_mcar5ORint /*
		*/ lo_mar1ORtre lo_mar1ORage lo_mar1ORint lo_mar2ORtre lo_mar2ORage lo_mar2ORint lo_mar3ORtre lo_mar3ORage lo_mar3ORint /*
		*/ lo_mar4ORtre lo_mar4ORage lo_mar4ORint lo_mar5ORtre lo_mar5ORage lo_mar5ORint /*
		*/ lo_mnar1ORtre lo_mnar1ORage lo_mnar1ORint lo_mnar2ORtre lo_mnar2ORage lo_mnar2ORint lo_mnar3ORtre lo_mnar3ORage lo_mnar3ORint /*
		*/ lo_mnar4ORtre lo_mnar4ORage lo_mnar4ORint lo_mnar5ORtre lo_mnar5ORage lo_mnar5ORint
		matrix mat4 = J(`msz',16,.)
		matrix colnames mat4 = ROC mcar1ROC mcar2ROC mcar3ROC mcar4ROC mcar5ROC mar1ROC mar2ROC mar3ROC mar4ROC mar5ROC /*
		*/ mnar1ROC mnar2ROC mnar3ROC mnar4ROC mnar5ROC
		matrix mat5 = J(`msz',60,.)
		matrix colnames mat5 = mcar6ORtre mcar6ORage mcar6ORint up_mcar6ORtre up_mcar6ORage up_mcar6ORint lo_mcar6ORtre lo_mcar6ORage lo_mcar6ORint mcar6ROC /*
		*/ mcar7ORtre mcar7ORage mcar7ORint up_mcar7ORtre up_mcar7ORage up_mcar7ORint lo_mcar7ORtre lo_mcar7ORage lo_mcar7ORint mcar7ROC /*
		*/ mar6ORtre mar6ORage mar6ORint up_mar6ORtre up_mar6ORage up_mar6ORint lo_mar6ORtre lo_mar6ORage lo_mar6ORint mar6ROC /*
		*/ mar7ORtre mar7ORage mar7ORint up_mar7ORtre up_mar7ORage up_mar7ORint lo_mar7ORtre lo_mar7ORage lo_mar7ORint mar7ROC /*
		*/ mnar6ORtre mnar6ORage mnar6ORint up_mnar6ORtre up_mnar6ORage up_mnar6ORint lo_mnar6ORtre lo_mnar6ORage lo_mnar6ORint mnar6ROC /*
		*/ mnar7ORtre mnar7ORage mnar7ORint up_mnar7ORtre up_mnar7ORage up_mnar7ORint lo_mnar7ORtre lo_mnar7ORage lo_mnar7ORint mnar7ROC
		
		//matrix counter
		local mctr=0		
		//replication loop
		forvalues i=1(1)`N' {
			local mctr=`mctr'+1
			qui {
			clear
			matrix C = (1, .5 \ .5, 1)
			drawnorm x age, n(`cases') corr(C)
			//no need to edit "age" to make its mean and sd recognisable. it affects the associations with the outcome and does not add much to the hypothesis
			//generate treatment
			gen treatment=0
			replace treatment=1 if x>0
			//generate outcome Pr(Y=1|group=0)=0.1 / Pr(Y=1|group=1)=0.18182 (for OR=2)
			gen outcome = uniform() < invlogit(-2.19722 + ln(2)*treatment + ln(1.5)*age + ln(1.2)*treatment*age)
			gen outcome2 = uniform() < invlogit(0 + ln(5)*outcome)
			qui save `tempx', replace

			//MCAR - X% for age - X% for outcome
			foreach x of varlist age outcome outcome2 {
				qui gen temp=runiform()
				qui replace `x'=. if temp<=`mrate'
				qui drop temp
			}
			qui save `tempf', replace
			qui keep if outcome!=.
			//no outcome prediction - include outcome in the model
			mi set flong
			mi register imputed age
			capture mi impute chained (regress) age = treatment outcome, augment add(10)
			if _rc==0 {
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat1[`mctr',5]=exp(_b[treatment])
					matrix mat1[`mctr',6]=exp(_b[age])
					matrix mat1[`mctr',7]=exp(_b[1.treatment#c.age])
					matrix mat2[`mctr',4]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat2[`mctr',5]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat2[`mctr',6]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat3[`mctr',4]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat3[`mctr',5]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat3[`mctr',6]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])	
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment age i.treatment#c.age"
					matrix mat4[`mctr',2]=e(b)
				}
			}
			//no outcome prediction - not include outcome in the model
			qui use `tempf', clear
			qui keep if outcome!=.
			mi set flong
			mi register imputed age
			capture mi impute chained (regress) age = treatment, augment add(10)
			if _rc==0 {			
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat1[`mctr',8]=exp(_b[treatment])
					matrix mat1[`mctr',9]=exp(_b[age])
					matrix mat1[`mctr',10]=exp(_b[1.treatment#c.age])
					matrix mat2[`mctr',7]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat2[`mctr',8]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat2[`mctr',9]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat3[`mctr',7]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat3[`mctr',8]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat3[`mctr',9]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment age i.treatment#c.age"
					matrix mat4[`mctr',3]=e(b)			
				}
			}
			//outcome prediction - include outcome in the model
			qui use `tempf', clear
			mi set flong
			mi register imputed age outcome
			capture mi impute chained (regress) age (logit) outcome = treatment, augment add(10)
			if _rc==0 {
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat1[`mctr',11]=exp(_b[treatment])
					matrix mat1[`mctr',12]=exp(_b[age])
					matrix mat1[`mctr',13]=exp(_b[1.treatment#c.age])
					matrix mat2[`mctr',10]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat2[`mctr',11]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat2[`mctr',12]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat3[`mctr',10]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat3[`mctr',11]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat3[`mctr',12]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment age i.treatment#c.age"
					matrix mat4[`mctr',4]=e(b)						
				}
			}
			//complete case analysis
			qui use `tempf', clear
			capture logit outcome treatment age i.treatment#c.age
			if _rc==0 {
				matrix mat1[`mctr',14]=exp(_b[treatment])
				matrix mat1[`mctr',15]=exp(_b[age])
				matrix mat1[`mctr',16]=exp(_b[1.treatment#c.age])
				matrix mat2[`mctr',13]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
				matrix mat2[`mctr',14]=exp(_b[age]+invnormal(0.975)*_se[age])
				matrix mat2[`mctr',15]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
				matrix mat3[`mctr',13]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
				matrix mat3[`mctr',14]=exp(_b[age]-invnormal(0.975)*_se[age])
				matrix mat3[`mctr',15]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])
				if "`txt'"=="ALL" {
					qui lroc, nograph
					matrix mat4[`mctr',5]=r(area)
				}
			}
			//mi with outcome-treatment but excluding age because of "high" missing data
			qui use `tempf', clear
			mi set flong
			mi register imputed outcome
			capture mi impute chained (logit) outcome = treatment, augment add(10)
			if _rc==0 {			
				capture  {
					quietly mi estimate, or post: logit outcome treatment
				}
				if _rc==0 {
					matrix mat1[`mctr',17]=exp(_b[treatment])
					matrix mat1[`mctr',18]=.
					matrix mat1[`mctr',19]=.
					matrix mat2[`mctr',16]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat2[`mctr',17]=.
					matrix mat2[`mctr',18]=.
					matrix mat3[`mctr',16]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat3[`mctr',17]=.
					matrix mat3[`mctr',18]=.
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment"
					matrix mat4[`mctr',6]=e(b)
				}
			}
			//no outcome prediction - include both outcomes in the model
			qui use `tempf', clear
			qui keep if outcome!=.
			mi set flong
			mi register imputed age outcome2
			capture mi impute chained (regress) age (logit) outcome2 = treatment outcome, augment add(10)
			if _rc==0 {
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat5[`mctr',1]=exp(_b[treatment])
					matrix mat5[`mctr',2]=exp(_b[age])
					matrix mat5[`mctr',3]=exp(_b[1.treatment#c.age])
					matrix mat5[`mctr',4]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat5[`mctr',5]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat5[`mctr',6]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat5[`mctr',7]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat5[`mctr',8]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat5[`mctr',9]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])	
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment age i.treatment#c.age"
					matrix mat5[`mctr',10]=e(b)
				}
			}
			//outcome prediction - include both outcomes in the model
			qui use `tempf', clear
			mi set flong
			mi register imputed age outcome outcome2
			capture mi impute chained (regress) age (logit) outcome outcome2 = treatment, augment add(10)
			if _rc==0 {
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat5[`mctr',11]=exp(_b[treatment])
					matrix mat5[`mctr',12]=exp(_b[age])
					matrix mat5[`mctr',13]=exp(_b[1.treatment#c.age])
					matrix mat5[`mctr',14]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat5[`mctr',15]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat5[`mctr',16]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat5[`mctr',17]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat5[`mctr',18]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat5[`mctr',19]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])	
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment age i.treatment#c.age"
					matrix mat5[`mctr',20]=e(b)						
				}
			}

			//MAR - X% for age (OR for age missingness=5 - with treatment), X% for outcome (OR for outcome missingness=5 - with treatment)
			qui use `tempx', clear
			foreach x of varlist age outcome outcome2 {
				if `mrate'<=0.5 {
					gen tempx=uniform() < invlogit(ln(5)*treatment)
				}
				else {
					gen tempx=uniform() < invlogit(ln(5)+ln(5)*treatment)
				}
				*limit missing where tempx=1 i.e. calculate updated missing rate
				qui count
				scalar denm=r(N)
				qui count if tempx==1
				scalar numm=r(N)
				scalar newmrate=`mrate'*denm/numm
				*final step
				qui gen tempy = uniform()
				qui replace `x'=. if tempy<newmrate & tempx==1
				qui drop tempx tempy
			}						
			qui save `tempf', replace
			qui keep if outcome!=.
			//no outcome prediction - include outcome in the model
			mi set flong
			mi register imputed age
			capture mi impute chained (regress) age = treatment outcome, augment add(10)
			if _rc==0 {	
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat1[`mctr',20]=exp(_b[treatment])
					matrix mat1[`mctr',21]=exp(_b[age])
					matrix mat1[`mctr',22]=exp(_b[1.treatment#c.age])
					matrix mat2[`mctr',19]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat2[`mctr',20]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat2[`mctr',21]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat3[`mctr',19]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat3[`mctr',20]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat3[`mctr',21]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])			
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment"
					matrix mat4[`mctr',7]=e(b)												
				}
			}
			//no outcome prediction - not include outcome in the model
			qui use `tempf', clear
			qui keep if outcome!=.
			mi set flong
			mi register imputed age
			capture mi impute chained (regress) age = treatment, augment add(10)
			if _rc==0 {
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat1[`mctr',23]=exp(_b[treatment])
					matrix mat1[`mctr',24]=exp(_b[age])
					matrix mat1[`mctr',25]=exp(_b[1.treatment#c.age])
					matrix mat2[`mctr',22]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat2[`mctr',23]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat2[`mctr',24]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat3[`mctr',22]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat3[`mctr',23]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat3[`mctr',24]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])	
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment"
					matrix mat4[`mctr',8]=e(b)												
				}
			}
			//outcome prediction - include outcome in the model
			qui use `tempf', clear
			mi set flong
			mi register imputed age outcome
			capture mi impute chained (regress) age (logit) outcome = treatment, augment add(10)
			if _rc==0 {
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat1[`mctr',26]=exp(_b[treatment])
					matrix mat1[`mctr',27]=exp(_b[age])
					matrix mat1[`mctr',28]=exp(_b[1.treatment#c.age])
					matrix mat2[`mctr',25]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat2[`mctr',26]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat2[`mctr',27]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat3[`mctr',25]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat3[`mctr',26]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat3[`mctr',27]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])	
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment"
					matrix mat4[`mctr',9]=e(b)												
				}
			}
			//complete case analysis
			qui use `tempf', clear
			capture logit outcome treatment age i.treatment#c.age
			if _rc==0 {
				matrix mat1[`mctr',29]=exp(_b[treatment])
				matrix mat1[`mctr',30]=exp(_b[age])
				matrix mat1[`mctr',31]=exp(_b[1.treatment#c.age])
				matrix mat2[`mctr',28]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
				matrix mat2[`mctr',29]=exp(_b[age]+invnormal(0.975)*_se[age])
				matrix mat2[`mctr',30]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
				matrix mat3[`mctr',28]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
				matrix mat3[`mctr',29]=exp(_b[age]-invnormal(0.975)*_se[age])
				matrix mat3[`mctr',30]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])	
				if "`txt'"=="ALL" {
					qui lroc, nograph
					matrix mat4[`mctr',10]=r(area)
				}
			}
			//mi with outcome-treatment but excluding age because of "high" missing data
			qui use `tempf', clear
			mi set flong
			mi register imputed outcome
			capture mi impute chained (logit) outcome = treatment, augment add(10)
			if _rc==0 {
				capture  {
					quietly mi estimate, or post: logit outcome treatment
				}
				if _rc==0 {
					matrix mat1[`mctr',32]=exp(_b[treatment])
					matrix mat1[`mctr',33]=.
					matrix mat1[`mctr',34]=.
					matrix mat2[`mctr',31]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat2[`mctr',32]=.
					matrix mat2[`mctr',33]=.
					matrix mat3[`mctr',31]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat3[`mctr',32]=.
					matrix mat3[`mctr',33]=.
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment"
					matrix mat4[`mctr',11]=e(b)												
				}
			}
			//no outcome prediction - include both outcomes in the model
			qui use `tempf', clear
			qui keep if outcome!=.
			mi set flong
			mi register imputed age outcome2
			capture mi impute chained (regress) age (logit) outcome2 = treatment outcome, augment add(10)
			if _rc==0 {
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat5[`mctr',21]=exp(_b[treatment])
					matrix mat5[`mctr',22]=exp(_b[age])
					matrix mat5[`mctr',23]=exp(_b[1.treatment#c.age])
					matrix mat5[`mctr',24]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat5[`mctr',25]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat5[`mctr',26]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat5[`mctr',27]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat5[`mctr',28]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat5[`mctr',29]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])	
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment age i.treatment#c.age"
					matrix mat5[`mctr',30]=e(b)
				}
			}
			//outcome prediction - include both outcome in the model
			qui use `tempf', clear
			mi set flong
			mi register imputed age outcome outcome2
			capture mi impute chained (regress) age (logit) outcome outcome2 = treatment, augment add(10)
			if _rc==0 {
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat5[`mctr',31]=exp(_b[treatment])
					matrix mat5[`mctr',32]=exp(_b[age])
					matrix mat5[`mctr',33]=exp(_b[1.treatment#c.age])
					matrix mat5[`mctr',34]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat5[`mctr',35]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat5[`mctr',36]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat5[`mctr',37]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat5[`mctr',38]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat5[`mctr',39]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])	
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment age i.treatment#c.age"
					matrix mat5[`mctr',40]=e(b)						
				}
			}

			//MNAR - X% for age (OR for age missingness=5 - with age), X% for outcome (OR for outcome missingness=5 - with outcome)
			qui use `tempx', clear
			foreach x of varlist age outcome outcome2 {	
				if `mrate'<=0.5 {
					gen tempx=uniform() < invlogit(ln(5)*`x')
				}
				else {
					gen tempx=uniform() < invlogit(ln(5)+ln(5)*`x')
				}
				*limit missing where tempx=1 i.e. calculate updated missing rate
				qui count
				scalar denm=r(N)
				qui count if tempx==1
				scalar numm=r(N)
				scalar newmrate=`mrate'*denm/numm
				*final step
				qui gen tempy = uniform()
				qui replace `x'=. if tempy<newmrate & tempx==1
				qui drop tempx tempy
			}
			qui save `tempf', replace
			qui keep if outcome!=.
			//no outcome prediction - include outcome in the model
			mi set flong
			mi register imputed age
			capture mi impute chained (regress) age = treatment outcome, augment add(10)
			if _rc==0 {
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat1[`mctr',35]=exp(_b[treatment])
					matrix mat1[`mctr',36]=exp(_b[age])
					matrix mat1[`mctr',37]=exp(_b[1.treatment#c.age])
					matrix mat2[`mctr',34]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat2[`mctr',35]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat2[`mctr',36]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat3[`mctr',34]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat3[`mctr',35]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat3[`mctr',36]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])	
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment"
					matrix mat4[`mctr',12]=e(b)				
				}
			}
			//no outcome prediction - not include outcome in the model
			qui use `tempf', clear
			qui keep if outcome!=.
			mi set flong
			mi register imputed age
			capture mi impute chained (regress) age = treatment, augment add(10)
			if _rc==0 {
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat1[`mctr',38]=exp(_b[treatment])
					matrix mat1[`mctr',39]=exp(_b[age])
					matrix mat1[`mctr',40]=exp(_b[1.treatment#c.age])
					matrix mat2[`mctr',37]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat2[`mctr',38]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat2[`mctr',39]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat3[`mctr',37]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat3[`mctr',38]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat3[`mctr',39]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])	
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment"
					matrix mat4[`mctr',13]=e(b)				
				}
			}
			//outcome prediction - include outcome in the model
			qui use `tempf', clear
			mi set flong
			mi register imputed age outcome
			capture mi impute chained (regress) age (logit) outcome = treatment, augment add(10)
			if _rc==0 {
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat1[`mctr',41]=exp(_b[treatment])
					matrix mat1[`mctr',42]=exp(_b[age])
					matrix mat1[`mctr',43]=exp(_b[1.treatment#c.age])
					matrix mat2[`mctr',40]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat2[`mctr',41]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat2[`mctr',42]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat3[`mctr',40]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat3[`mctr',41]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat3[`mctr',42]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])	
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment"
					matrix mat4[`mctr',14]=e(b)				
				}
			}
			//complete case analysis
			qui use `tempf', clear
			capture logit outcome treatment age i.treatment#c.age
			if _rc==0 {
				matrix mat1[`mctr',44]=exp(_b[treatment])
				matrix mat1[`mctr',45]=exp(_b[age])
				matrix mat1[`mctr',46]=exp(_b[1.treatment#c.age])
				matrix mat2[`mctr',43]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
				matrix mat2[`mctr',44]=exp(_b[age]+invnormal(0.975)*_se[age])
				matrix mat2[`mctr',45]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
				matrix mat3[`mctr',43]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
				matrix mat3[`mctr',44]=exp(_b[age]-invnormal(0.975)*_se[age])
				matrix mat3[`mctr',45]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])
				if "`txt'"=="ALL" {
					qui lroc, nograph
					matrix mat4[`mctr',15]=r(area)
				}
			}
			//mi with outcome-treatment but excluding age because of "high" missing data
			qui use `tempf', clear
			mi set flong
			mi register imputed outcome
			capture mi impute chained (logit) outcome = treatment, augment add(10)
			if _rc==0 {
				capture  {
					quietly mi estimate, or post: logit outcome treatment
				}
				if _rc==0 {
					matrix mat1[`mctr',47]=exp(_b[treatment])
					matrix mat1[`mctr',48]=.
					matrix mat1[`mctr',49]=.
					matrix mat2[`mctr',46]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat2[`mctr',47]=.
					matrix mat2[`mctr',48]=.
					matrix mat3[`mctr',46]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat3[`mctr',47]=.
					matrix mat3[`mctr',48]=.
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment"
					matrix mat4[`mctr',16]=e(b)				
				}
			}
			//no outcome prediction - include both outcomes in the model
			qui use `tempf', clear
			qui keep if outcome!=.			
			mi set flong
			mi register imputed age outcome2
			capture mi impute chained (regress) age (logit) outcome2 = treatment outcome, augment add(10)
			if _rc==0 {
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat5[`mctr',41]=exp(_b[treatment])
					matrix mat5[`mctr',42]=exp(_b[age])
					matrix mat5[`mctr',43]=exp(_b[1.treatment#c.age])
					matrix mat5[`mctr',44]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat5[`mctr',45]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat5[`mctr',46]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat5[`mctr',47]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat5[`mctr',48]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat5[`mctr',49]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])	
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment age i.treatment#c.age"
					matrix mat5[`mctr',50]=e(b)
				}
			}
			//outcome prediction - include both outcome in the model
			qui use `tempf', clear
			mi set flong
			mi register imputed age outcome outcome2
			capture mi impute chained (regress) age (logit) outcome outcome2 = treatment, augment add(10)
			if _rc==0 {
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat5[`mctr',51]=exp(_b[treatment])
					matrix mat5[`mctr',52]=exp(_b[age])
					matrix mat5[`mctr',53]=exp(_b[1.treatment#c.age])
					matrix mat5[`mctr',54]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat5[`mctr',55]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat5[`mctr',56]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat5[`mctr',57]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat5[`mctr',58]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat5[`mctr',59]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])	
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment age i.treatment#c.age"
					matrix mat5[`mctr',60]=e(b)						
				}
			}
			} /*qui close*/
			di in yellow "." _continue
			if mod(`i',50)==0 {
				di in yellow `i'
			}
			//save tempfiles if large
			if mod(`i',`msizelim')==0 {
				local mctr=0
				clear
				qui svmat mat1, names(col)
				qui svmat mat2, names(col)
				qui svmat mat3, names(col)
				qui svmat mat4, names(col)
				qui svmat mat5, names(col)					
				qui drop if mcar1ORtre==.
				capture save `temph'
				if _rc!=0 {
					qui append using `temph'
					qui save `temph', replace
				}
				//reset matrices
				matrix mat1 = J(`msz',49,.)
				matrix colnames mat1 = ORtre ORage ORint corr /*
				*/ mcar1ORtre mcar1ORage mcar1ORint mcar2ORtre mcar2ORage mcar2ORint mcar3ORtre mcar3ORage mcar3ORint /*
				*/ mcar4ORtre mcar4ORage mcar4ORint mcar5ORtre mcar5ORage mcar5ORint /*
				*/ mar1ORtre mar1ORage mar1ORint mar2ORtre mar2ORage mar2ORint mar3ORtre mar3ORage mar3ORint /*
				*/ mar4ORtre mar4ORage mar4ORint mar5ORtre mar5ORage mar5ORint /*
				*/ mnar1ORtre mnar1ORage mnar1ORint mnar2ORtre mnar2ORage mnar2ORint mnar3ORtre mnar3ORage mnar3ORint /*
				*/ mnar4ORtre mnar4ORage mnar4ORint mnar5ORtre mnar5ORage mnar5ORint
				matrix mat2 = J(`msz',48,.)
				matrix colnames mat2 = up_ORtre up_ORage up_ORint /*
				*/ up_mcar1ORtre up_mcar1ORage up_mcar1ORint up_mcar2ORtre up_mcar2ORage up_mcar2ORint up_mcar3ORtre up_mcar3ORage up_mcar3ORint /*
				*/ up_mcar4ORtre up_mcar4ORage up_mcar4ORint up_mcar5ORtre up_mcar5ORage up_mcar5ORint /*
				*/ up_mar1ORtre up_mar1ORage up_mar1ORint up_mar2ORtre up_mar2ORage up_mar2ORint up_mar3ORtre up_mar3ORage up_mar3ORint /*
				*/ up_mar4ORtre up_mar4ORage up_mar4ORint up_mar5ORtre up_mar5ORage up_mar5ORint /*
				*/ up_mnar1ORtre up_mnar1ORage up_mnar1ORint up_mnar2ORtre up_mnar2ORage up_mnar2ORint up_mnar3ORtre up_mnar3ORage up_mnar3ORint /*
				*/ up_mnar4ORtre up_mnar4ORage up_mnar4ORint up_mnar5ORtre up_mnar5ORage up_mnar5ORint
				matrix mat3 = J(`msz',48,.)
				matrix colnames mat3 = lo_ORtre lo_ORage lo_ORint /*
				*/ lo_mcar1ORtre lo_mcar1ORage lo_mcar1ORint lo_mcar2ORtre lo_mcar2ORage lo_mcar2ORint lo_mcar3ORtre lo_mcar3ORage lo_mcar3ORint /*
				*/ lo_mcar4ORtre lo_mcar4ORage lo_mcar4ORint lo_mcar5ORtre lo_mcar5ORage lo_mcar5ORint /*
				*/ lo_mar1ORtre lo_mar1ORage lo_mar1ORint lo_mar2ORtre lo_mar2ORage lo_mar2ORint lo_mar3ORtre lo_mar3ORage lo_mar3ORint /*
				*/ lo_mar4ORtre lo_mar4ORage lo_mar4ORint lo_mar5ORtre lo_mar5ORage lo_mar5ORint /*
				*/ lo_mnar1ORtre lo_mnar1ORage lo_mnar1ORint lo_mnar2ORtre lo_mnar2ORage lo_mnar2ORint lo_mnar3ORtre lo_mnar3ORage lo_mnar3ORint /*
				*/ lo_mnar4ORtre lo_mnar4ORage lo_mnar4ORint lo_mnar5ORtre lo_mnar5ORage lo_mnar5ORint
				matrix mat4 = J(`msz',16,.)
				matrix colnames mat4 = ROC mcar1ROC mcar2ROC mcar3ROC mcar4ROC mcar5ROC mar1ROC mar2ROC mar3ROC mar4ROC mar5ROC /*
				*/ mnar1ROC mnar2ROC mnar3ROC mnar4ROC mnar5ROC
				matrix mat5 = J(`msz',60,.)
				matrix colnames mat5 = mcar6ORtre mcar6ORage mcar6ORint up_mcar6ORtre up_mcar6ORage up_mcar6ORint lo_mcar6ORtre lo_mcar6ORage lo_mcar6ORint mcar6ROC /*
				*/ mcar7ORtre mcar7ORage mcar7ORint up_mcar7ORtre up_mcar7ORage up_mcar7ORint lo_mcar7ORtre lo_mcar7ORage lo_mcar7ORint mcar7ROC /*
				*/ mar6ORtre mar6ORage mar6ORint up_mar6ORtre up_mar6ORage up_mar6ORint lo_mar6ORtre lo_mar6ORage lo_mar6ORint mar6ROC /*
				*/ mar7ORtre mar7ORage mar7ORint up_mar7ORtre up_mar7ORage up_mar7ORint lo_mar7ORtre lo_mar7ORage lo_mar7ORint mar7ROC /*
				*/ mnar6ORtre mnar6ORage mnar6ORint up_mnar6ORtre up_mnar6ORage up_mnar6ORint lo_mnar6ORtre lo_mnar6ORage lo_mnar6ORint mnar6ROC /*
				*/ mnar7ORtre mnar7ORage mnar7ORint up_mnar7ORtre up_mnar7ORage up_mnar7ORint lo_mnar7ORtre lo_mnar7ORage lo_mnar7ORint mnar7ROC				
			}			
		}
		clear
		qui svmat mat1, names(col)
		qui svmat mat2, names(col)
		qui svmat mat3, names(col)
		qui svmat mat4, names(col)
		qui svmat mat5, names(col)
		qui drop if mcar1ORtre==.
		if `N'!=`msz' {
			qui append using `temph'
		}		
		if "`txt'"!="ALL" {
			qui drop *ROC*
		}
		qui compress
		qui save dataset_`txt'_`N'_`cases'_`=string(round(`mrate',0.1))'.dta ,replace
	}
end

//AUC calculation program from mi logit
cap program drop elroc
program elroc, eclass
	tempname b
	tempfile tempx
	qui save `tempx', replace
	/* Step 1: perform ROC analysis */
	args varbls
	local marea=0
	local cntr=0
	forvalues i=1(1)10 {
		qui use `tempx', clear
		mi extract `i', clear
		capture logit `varbls'
		if _rc==0 {
			qui lroc, nograph
			local cntr=`cntr'+1
			local marea = `marea'+r(area)
		}
	}
	qui use `tempx', clear
	/* Step 2: save estimate in temporary matrices*/
	tempname b
	if `cntr'>0 {
		mat `b' = `marea'/`cntr'
	}
	else {
		mat `b' = .
	}
	local N = `cntr'
	/* Step 3: make column names and row names consistent*/
	mat colnames `b' = AUC
	/*Step 4: post results to e()*/
	ereturn post `b', obs(`N')
	ereturn local cmd "elroc"
	ereturn local title "ROC area"
end

