/*assume:
10000 patients
binary outcome
binary treatment 50-50
continuous covariate confounded with treatment
*/
capture program drop addsim
program addsim
	//cd "P:\Evan\Methodological\Outcome MI\"
	//cd "D:\Dropbox\work\temp\Outcome MI\"
	//cd "/mnt/iusers01/ja01/mdmysek3/Stata/OutcomeMI/"
	set more off
	set maxiter 30
	tempfile tempx tempf tempg temph
	set seed 7
	set matsize 10000
	//inputs
	local N=`1'		/*number of replications*/
	local cases=`2'	/*number of cases in each dataset*/
	local msizelim=10
	//decide on matrix size
	local msz=`N'
	if `N'>`msizelim' {
		local msz=`msizelim'
	}	
	
	//allow roc with argument
	local txt="noROC"
	if "`1'"=="roc" {
		local txt="ALL"
	}

	//missing rate loop
	forvalues mrate=0.8(0.2)0.8 {
		di in green _newline(2) "`txt' cases=`cases' missrate=`mrate':"
		//local mrate=0.2
		qui clear
		//matrices
		matrix mat1 = J(`msz',30,.)
		matrix colnames mat1 = mcar8ORtre mcar8ORage mcar8ORint up_mcar8ORtre up_mcar8ORage up_mcar8ORint lo_mcar8ORtre lo_mcar8ORage lo_mcar8ORint mcar8ROC /*
		*/ mar8ORtre mar8ORage mar8ORint up_mar8ORtre up_mar8ORage up_mar8ORint lo_mar8ORtre lo_mar8ORage lo_mar8ORint mar8ROC /*
		*/ mnar8ORtre mnar8ORage mnar8ORint up_mnar8ORtre up_mnar8ORage up_mnar8ORint lo_mnar8ORtre lo_mnar8ORage lo_mnar8ORint mnar8ROC
		
		//matrix counter
		local mctr=0
		//replication loop
		forvalues i=1(1)`N' {
			local mctr=`mctr'+1
			qui {
			clear
			matrix C = (1, .5 \ .5, 1)
			drawnorm x age, n(`cases') corr(C)
			//no need to edit "age" to make its mean and sd recognisable. it affects the associations with the outcome and does not add much to the hypothesis
			//generate treatment
			gen treatment=0
			replace treatment=1 if x>0
			//generate outcome Pr(Y=1|group=0)=0.1 / Pr(Y=1|group=1)=0.18182 (for OR=2)
			gen outcome = uniform() < invlogit(-2.19722 + ln(2)*treatment + ln(1.5)*age + ln(1.2)*treatment*age)
			gen outcome2 = uniform() < invlogit(0 + ln(5)*outcome)
			egen id = seq()
			qui save `tempx', replace

			//MCAR - X% for age - X% for outcome
			foreach x of varlist age outcome outcome2 {
				qui gen temp=runiform()
				qui replace `x'=. if temp<=`mrate'
				qui drop temp
			}
			qui save `tempf', replace
			qui keep if outcome!=.
			qui keep id
			qui save `tempg', replace
			//outcome prediction - include outcome in the model
			qui use `tempf', clear
			mi set flong
			mi register imputed age outcome
			capture mi impute chained (regress) age (logit) outcome = treatment, augment add(10)
			if _rc==0 {
				//merge with original data and keep only where outcome not missing
				qui merge m:1 id using `tempg'
				qui keep if _merge==3			
				qui mi update
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat1[`mctr',1]=exp(_b[treatment])
					matrix mat1[`mctr',2]=exp(_b[age])
					matrix mat1[`mctr',3]=exp(_b[1.treatment#c.age])
					matrix mat1[`mctr',4]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat1[`mctr',5]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat1[`mctr',6]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat1[`mctr',7]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat1[`mctr',8]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat1[`mctr',9]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment age i.treatment#c.age"
					matrix mat1[`mctr',10]=e(b)						
				}
			}

			//MAR - X% for age (OR for age missingness=5 - with treatment), X% for outcome (OR for outcome missingness=5 - with treatment)
			qui use `tempx', clear
			foreach x of varlist age outcome outcome2 {
				if `mrate'<=0.5 {
					gen tempx=uniform() < invlogit(ln(5)*treatment)
				}
				else {
					gen tempx=uniform() < invlogit(ln(5)+ln(5)*treatment)
				}				
				*limit missing where tempx=1 i.e. calculate updated missing rate
				qui count
				scalar denm=r(N)
				qui count if tempx==1
				scalar numm=r(N)
				scalar newmrate=`mrate'*denm/numm
				*final step
				qui gen tempy = uniform()
				qui replace `x'=. if tempy<newmrate & tempx==1
				qui drop tempx tempy
			}						
			qui save `tempf', replace
			qui keep if outcome!=.
			qui keep id
			qui save `tempg', replace
			//outcome prediction - include outcome in the model
			qui use `tempf', clear
			mi set flong
			mi register imputed age outcome
			capture mi impute chained (regress) age (logit) outcome = treatment, augment add(10)
			if _rc==0 {
				//merge with original data and keep only where outcome not missing
				qui merge m:1 id using `tempg'
				qui keep if _merge==3			
				qui mi update
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat1[`mctr',11]=exp(_b[treatment])
					matrix mat1[`mctr',12]=exp(_b[age])
					matrix mat1[`mctr',13]=exp(_b[1.treatment#c.age])
					matrix mat1[`mctr',14]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat1[`mctr',15]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat1[`mctr',16]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat1[`mctr',17]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat1[`mctr',18]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat1[`mctr',19]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment age i.treatment#c.age"
					matrix mat1[`mctr',20]=e(b)						
				}
			}
			
			//MNAR - X% for age (OR for age missingness=5 - with age), X% for outcome (OR for outcome missingness=5 - with outcome)
			qui use `tempx', clear
			foreach x of varlist age outcome outcome2 {			
				if `mrate'<=0.5 {
					gen tempx=uniform() < invlogit(ln(5)*`x')
				}
				else {
					gen tempx=uniform() < invlogit(ln(5)+ln(5)*`x')
				}				
				*limit missing where tempx=1 i.e. calculate updated missing rate
				qui count
				scalar denm=r(N)
				qui count if tempx==1
				scalar numm=r(N)
				scalar newmrate=`mrate'*denm/numm
				*final step
				qui gen tempy = uniform()
				qui replace `x'=. if tempy<newmrate & tempx==1
				qui drop tempx tempy
			}
			qui save `tempf', replace
			qui keep if outcome!=.
			qui keep id
			qui save `tempg', replace
			//outcome prediction - include outcome in the model
			qui use `tempf', clear
			mi set flong
			mi register imputed age outcome
			capture mi impute chained (regress) age (logit) outcome = treatment, augment add(10)
			if _rc==0 {
				//merge with original data and keep only where outcome not missing
				qui merge m:1 id using `tempg'
				qui keep if _merge==3			
				qui mi update
				capture  {
					quietly mi estimate, or post: logit outcome treatment age i.treatment#c.age
				}
				if _rc==0 {
					matrix mat1[`mctr',21]=exp(_b[treatment])
					matrix mat1[`mctr',22]=exp(_b[age])
					matrix mat1[`mctr',23]=exp(_b[1.treatment#c.age])
					matrix mat1[`mctr',24]=exp(_b[treatment]+invnormal(0.975)*_se[treatment])
					matrix mat1[`mctr',25]=exp(_b[age]+invnormal(0.975)*_se[age])
					matrix mat1[`mctr',26]=exp(_b[1.treatment#c.age]+invnormal(0.975)*_se[1.treatment#c.age])
					matrix mat1[`mctr',27]=exp(_b[treatment]-invnormal(0.975)*_se[treatment])
					matrix mat1[`mctr',28]=exp(_b[age]-invnormal(0.975)*_se[age])
					matrix mat1[`mctr',29]=exp(_b[1.treatment#c.age]-invnormal(0.975)*_se[1.treatment#c.age])
				}
				if "`txt'"=="ALL" {
					elroc "outcome treatment age i.treatment#c.age"
					matrix mat1[`mctr',30]=e(b)						
				}
			}
			} /*qui close*/
			di in yellow "." _continue
			if mod(`i',50)==0 {
				di in yellow `i'
			}
			//save tempfiles if large
			if mod(`i',`msizelim')==0 {
				local mctr=0
				clear
				qui svmat mat1, names(col)				
				qui drop if mcar8ORtre==.
				capture save `temph'
				if _rc!=0 {
					qui append using `temph'
					qui save `temph', replace
				}
				matrix mat1 = J(`msz',30,.)
				matrix colnames mat1 = mcar8ORtre mcar8ORage mcar8ORint up_mcar8ORtre up_mcar8ORage up_mcar8ORint lo_mcar8ORtre lo_mcar8ORage lo_mcar8ORint mcar8ROC /*
				*/ mar8ORtre mar8ORage mar8ORint up_mar8ORtre up_mar8ORage up_mar8ORint lo_mar8ORtre lo_mar8ORage lo_mar8ORint mar8ROC /*
				*/ mnar8ORtre mnar8ORage mnar8ORint up_mnar8ORtre up_mnar8ORage up_mnar8ORint lo_mnar8ORtre lo_mnar8ORage lo_mnar8ORint mnar8ROC				
			}
		}
		clear
		qui svmat mat1, names(col)
		qui drop if mcar8ORtre==.
		if `N'!=`msz' {
			qui append using `temph'
		}
		if "`txt'"!="ALL" {
			qui drop *ROC*
		}
		qui {
			capture drop id
			compress
			egen id=seq()
			save `tempx', replace
			use dataset_`txt'_`N'_`cases'_`=string(round(`mrate',0.1))'.dta ,replace
			egen id=seq()
			merge 1:1 id using `tempx'
			qui drop _merge
			order id
			save dataset_`txt'_`N'_`cases'_`=string(round(`mrate',0.1))'.dta ,replace
		}
	}
end

//AUC calculation program from mi logit
cap program drop elroc
program elroc, eclass
	tempname b
	tempfile tempx
	qui save `tempx', replace
	/* Step 1: perform ROC analysis */
	args varbls
	local marea=0
	local cntr=0
	forvalues i=1(1)10 {
		qui use `tempx', clear
		mi extract `i', clear
		capture logit `varbls'
		if _rc==0 {
			qui lroc, nograph
			local cntr=`cntr'+1
			local marea = `marea'+r(area)
		}
	}
	qui use `tempx', clear
	/* Step 2: save estimate in temporary matrices*/
	tempname b
	if `cntr'>0 {
		mat `b' = `marea'/`cntr'
	}
	else {
		mat `b' = .
	}
	local N = `cntr'
	/* Step 3: make column names and row names consistent*/
	mat colnames `b' = AUC
	/*Step 4: post results to e()*/
	ereturn post `b', obs(`N')
	ereturn local cmd "elroc"
	ereturn local title "ROC area"
end

