//get results programme
capture program drop getres
program getres
	cd "P:\Evan\Methodological\Outcome MI\"
	set more off
	//inputs
	local N=1000		/*number of datasets*/
	local cases=10000	/*in each dataset*/
	//power settings
	local powd1=2.0
	local powd2=1.5
	local powd3=1.2

	//loop for missingness
	foreach mv in 2 4 6 8 {
		qui use dataset_noROC_`N'_`cases'_.`mv'.dta, clear
		//do some cleaning - if extreme OR set to missing
		foreach x of varlist mcar*ORtre mar*ORtre mnar*ORtre mcar*ORage mar*ORage /*
		*/ mnar*ORage mcar*ORint mar*ORint mnar*ORint {
			qui gen temp=0
			qui replace temp=1 if (`x'>1000 | `x'<0.01) & `x'!=.
			qui replace temp=1 if `x'==1 & lo_`x'==1 & up_`x'==1	/*non convergence issues all estimates are=1*/
			//set to missing all model estimates
			local spos=strpos("`x'","OR")
			local y=substr("`x'",1,`=`spos'+1')			
			foreach z in tre age int {
				qui replace `y'`z'=. if temp==1
				qui replace lo_`y'`z'=. if temp==1
				qui replace up_`y'`z'=. if temp==1
			}
			qui drop temp					
		}
		//get some counts
		//MCAR MAR MNAR loop
		local i=0
		foreach y in "mcar" "mar" "mnar" {
			local i=`i'+1
			//methods loop
			forvalues j=1(1)8 {
				qui count if `y'`j'ORtre!=.
				scalar cnt_`i'`j'_`mv'=r(N)
			}
		}		
		//ABSOLUTE MEAN ERROR
		//MCAR
		forvalues i=1(1)8 {
			qui gen E_ME1`i' = abs(ln(2.0)-ln(mcar`i'ORtre))
			qui gen X_ME1`i' = abs(ln(1.5)-ln(mcar`i'ORage))
			qui gen I_ME1`i' = abs(ln(1.2)-ln(mcar`i'ORint))
		}
		//MAR
		forvalues i=1(1)8 {
			qui gen E_ME2`i' = abs(ln(2.0)-ln(mar`i'ORtre))
			qui gen X_ME2`i' = abs(ln(1.5)-ln(mar`i'ORage))
			qui gen I_ME2`i' = abs(ln(1.2)-ln(mar`i'ORint))
		}	
		//MNAR
		forvalues i=1(1)8 {
			qui gen E_ME3`i' = abs(ln(2.0)-ln(mnar`i'ORtre))
			qui gen X_ME3`i' = abs(ln(1.5)-ln(mnar`i'ORage))
			qui gen I_ME3`i' = abs(ln(1.2)-ln(mnar`i'ORint))
		}	
		//MEAN BIAS
		//MCAR
		forvalues i=1(1)8 {
			qui gen E_MB1`i' = ln(2.0)-ln(mcar`i'ORtre)
			qui gen X_MB1`i' = ln(1.5)-ln(mcar`i'ORage)
			qui gen I_MB1`i' = ln(1.2)-ln(mcar`i'ORint)
		}
		//MAR
		forvalues i=1(1)8 {
			qui gen E_MB2`i' = ln(2.0)-ln(mar`i'ORtre)
			qui gen X_MB2`i' = ln(1.5)-ln(mar`i'ORage)
			qui gen I_MB2`i' = ln(1.2)-ln(mar`i'ORint)
		}	
		//MNAR
		forvalues i=1(1)8 {
			qui gen E_MB3`i' = ln(2.0)-ln(mnar`i'ORtre)
			qui gen X_MB3`i' = ln(1.5)-ln(mnar`i'ORage)
			qui gen I_MB3`i' = ln(1.2)-ln(mnar`i'ORint)
		}	
		//COVERAGE
		//MCAR
		forvalues i=1(1)8 {
			qui gen E_CV1`i' = 0 if lo_mcar`i'ORtre!=.
			qui replace E_CV1`i' = 1 if lo_mcar`i'ORtre<2.0 & 2.0<up_mcar`i'ORtre
			qui gen X_CV1`i' = 0 if lo_mcar`i'ORage!=.
			qui replace X_CV1`i' = 1 if lo_mcar`i'ORage<1.5 & 1.5<up_mcar`i'ORage
			qui gen I_CV1`i' = 0 if lo_mcar`i'ORint!=.
			qui replace I_CV1`i' = 1 if lo_mcar`i'ORint<1.2 & 1.2<up_mcar`i'ORint
		}
		//MAR
		forvalues i=1(1)8 {
			qui gen E_CV2`i' = 0 if lo_mar`i'ORtre!=.
			qui replace E_CV2`i' = 1 if lo_mar`i'ORtre<2.0 & 2.0<up_mar`i'ORtre
			qui gen X_CV2`i' = 0 if lo_mar`i'ORage!=.
			qui replace X_CV2`i' = 1 if lo_mar`i'ORage<1.5 & 1.5<up_mar`i'ORage
			qui gen I_CV2`i' = 0 if lo_mar`i'ORint!=.
			qui replace I_CV2`i' = 1 if lo_mar`i'ORint<1.2 & 1.2<up_mar`i'ORint
		}	
		//MNAR
		forvalues i=1(1)8 {
			qui gen E_CV3`i' = 0 if lo_mnar`i'ORtre!=.
			qui replace E_CV3`i' = 1 if lo_mnar`i'ORtre<2.0 & 2.0<up_mnar`i'ORtre
			qui gen X_CV3`i' = 0 if lo_mnar`i'ORage!=.
			qui replace X_CV3`i' = 1 if lo_mnar`i'ORage<1.5 & 1.5<up_mnar`i'ORage
			qui gen I_CV3`i' = 0 if lo_mnar`i'ORint!=.
			qui replace I_CV3`i' = 1 if lo_mnar`i'ORint<1.2 & 1.2<up_mnar`i'ORint
		}		
		//POWER
		//MCAR
		forvalues i=1(1)8 {
			qui gen E_PO1`i' = 1 if lo_mcar`i'ORtre!=.
			qui replace E_PO1`i' = 0 if lo_mcar`i'ORtre<`=2.0-`powd1'' & up_mcar`i'ORtre>`=2.0-`powd1''
			qui replace E_PO1`i' = 0 if lo_mcar`i'ORtre<`=2.0+`powd1'' & up_mcar`i'ORtre>`=2.0+`powd1''
			qui gen X_PO1`i' = 1 if lo_mcar`i'ORage!=.
			qui replace X_PO1`i' = 0 if lo_mcar`i'ORage<`=1.5-`powd2'' & up_mcar`i'ORage>`=1.5-`powd2''
			qui replace X_PO1`i' = 0 if lo_mcar`i'ORage<`=1.5+`powd2'' & up_mcar`i'ORage>`=1.5+`powd2''
			qui gen I_PO1`i' = 1 if lo_mcar`i'ORint!=.
			qui replace I_PO1`i' = 0 if lo_mcar`i'ORint<`=1.2-`powd3'' & up_mcar`i'ORint>`=1.2-`powd3''
			qui replace I_PO1`i' = 0 if lo_mcar`i'ORint<`=1.2+`powd3'' & up_mcar`i'ORint>`=1.2+`powd3''
		}
		//MAR
		forvalues i=1(1)8 {
			qui gen E_PO2`i' = 1 if lo_mar`i'ORtre!=.
			qui replace E_PO2`i' = 0 if lo_mar`i'ORtre<`=2.0-`powd1'' & up_mar`i'ORtre>`=2.0-`powd1''
			qui replace E_PO2`i' = 0 if lo_mar`i'ORtre<`=2.0+`powd1'' & up_mar`i'ORtre>`=2.0+`powd1''
			qui gen X_PO2`i' = 1 if lo_mar`i'ORage!=.
			qui replace X_PO2`i' = 0 if lo_mar`i'ORage<`=1.5-`powd2'' & up_mar`i'ORage>`=1.5-`powd2''
			qui replace X_PO2`i' = 0 if lo_mar`i'ORage<`=1.5+`powd2'' & up_mar`i'ORage>`=1.5+`powd2''
			qui gen I_PO2`i' = 1 if lo_mar`i'ORint!=.
			qui replace I_PO2`i' = 0 if lo_mar`i'ORint<`=1.2-`powd3'' & up_mar`i'ORint>`=1.2-`powd3''
			qui replace I_PO2`i' = 0 if lo_mar`i'ORint<`=1.2+`powd3'' & up_mar`i'ORint>`=1.2+`powd3''
		}	
		//MNAR
		forvalues i=1(1)8 {
			qui gen E_PO3`i' = 1 if lo_mnar`i'ORtre!=.
			qui replace E_PO3`i' = 0 if lo_mnar`i'ORtre<`=2.0-`powd1'' & up_mnar`i'ORtre>`=2.0-`powd1''
			qui replace E_PO3`i' = 0 if lo_mnar`i'ORtre<`=2.0+`powd1'' & up_mnar`i'ORtre>`=2.0+`powd1''
			qui gen X_PO3`i' = 1 if lo_mnar`i'ORage!=.
			qui replace X_PO3`i' = 0 if lo_mnar`i'ORage<`=1.5-`powd2'' & up_mnar`i'ORage>`=1.5-`powd2''
			qui replace X_PO3`i' = 0 if lo_mnar`i'ORage<`=1.5+`powd2'' & up_mnar`i'ORage>`=1.5+`powd2''
			qui gen I_PO3`i' = 1 if lo_mnar`i'ORint!=.
			qui replace I_PO3`i' = 0 if lo_mnar`i'ORint<`=1.2-`powd3'' & up_mnar`i'ORint>`=1.2-`powd3''
			qui replace I_PO3`i' = 0 if lo_mnar`i'ORint<`=1.2+`powd3'' & up_mnar`i'ORint>`=1.2+`powd3''
		}			
		
		//summarise
		local txME "r(mean)" /*"r(p50)"*/
		local txMB "r(mean)" /*"r(p50)"*/
		local txCV "r(mean)"
		local txPO "r(mean)"
		foreach x in "ME" "MB" "CV" "PO" {
			//MCAR MAR MNAR loop
			forvalues i=1(1)3 {
				//methods loop
				forvalues j=1(1)8 {
					//predictor loop
					foreach y in E X I {
						qui sum `y'_`x'`i'`j', detail
						scalar v`y'_`x'`i'`j'_`mv'=`tx`x''
						//if binary
						if r(max)==1 & r(min)==0 {
							scalar se`y'_`x'`i'`j'_`mv'=sqrt(r(mean)*(1-r(mean)))/sqrt(`N')
						}
						else {
							scalar se`y'_`x'`i'`j'_`mv'=r(sd)/sqrt(`N')
						}

					}
				}
			}
		}
		//set CV to percentage
		forvalues i=1(1)3 {
			forvalues j=1(1)3 {
				//scalar vE_CV`i'`j'_`mv'=vE_CV`i'`j'_`mv'*100
				//scalar vX_CV`i'`j'_`mv'=vX_CV`i'`j'_`mv'*100
				//scalar vI_CV`i'`j'_`mv'=vI_CV`i'`j'_`mv'*100
			}
		}		
	}
	
	
	//report
    qui log using "3-summary_tables_N=`N'_cases=`cases'", name(log1) smcl replace
	qui log using "3-summary_tables_N=`N'_cases=`cases'", name(log2) text replace
	
	/*display*/
    set linesize 160
    /*generate tables*/
    local cl1 = 18
    forvalues i=2(1)15 {
        local cl`i' = `cl`=`i'-1'' + 9
    }
	
	di _newline(2) "Exposure table"
	di _col(2) "analysis A: complete case analysis"
	di _col(2) "analysis B: mi with outcome NOT included in model (and not predicted)"
	di _col(2) "analysis C: mi with outcome included in model but NOT predicted"
	di _col(2) "analysis D: mi with outcome included in model AND predicted"
	di _col(2) "analysis E: mi with outcome included in model AND predicted but then deleted"
	di _col(2) "analysis F: as in C but also including outcome2 in mi model"
	di _col(2) "analysis G: as in D but also including outcome2 in mi model"
	di _col(2) "analysis H: as in D but without the covariate"
	//cols
    di in green _col(`cl1') "A" _col(`cl2') "B" _col(`cl3') "C" _col(`cl4') "D" _col(`cl5') "E" _col(`cl6') "F" _col(`cl7') "G" _col(`cl8') "H"
	//line output
	local i=0
	foreach y in "MCAR" "MAR" "MNAR" {
		local i=`i'+1
		foreach x in "MB" "ME" "CV" "PO" {
			foreach mv in "2" "4" "6" "8" {		
				di in green _col(2) "`y'" _col(10) "`x'" _col(14) ".`mv'" in yellow /*
				*/ _col(`cl1') %4.3f vE_`x'`i'4_`mv' _col(`cl2') %4.3f vE_`x'`i'2_`mv' _col(`cl3') %4.3f vE_`x'`i'1_`mv' _col(`cl4') %4.3f vE_`x'`i'3_`mv' /*
				*/ _col(`cl5') %4.3f vE_`x'`i'8_`mv' _col(`cl6') %4.3f vE_`x'`i'6_`mv' _col(`cl7') %4.3f vE_`x'`i'7_`mv' _col(`cl8') %4.3f vE_`x'`i'5_`mv'
			}
		}
	}	

	di _newline(2) "Covariate table"
	di _col(2) "analysis A: complete case analysis"
	di _col(2) "analysis B: mi with outcome NOT included in model (and not predicted)"
	di _col(2) "analysis C: mi with outcome included in model but NOT predicted"
	di _col(2) "analysis D: mi with outcome included in model AND predicted"
	di _col(2) "analysis E: mi with outcome included in model AND predicted but then deleted"
	di _col(2) "analysis F: as in C but also including outcome2 in mi model"
	di _col(2) "analysis G: as in D but also including outcome2 in mi model"
	di _col(2) "analysis H: as in D but without the covariate"
	//cols
    di in green _col(`cl1') "A" _col(`cl2') "B" _col(`cl3') "C" _col(`cl4') "D" _col(`cl5') "E" _col(`cl6') "F" _col(`cl7') "G" _col(`cl8') "H"
	//line output
	local i=0
	foreach y in "MCAR" "MAR" "MNAR" {
		local i=`i'+1
		foreach x in "MB" "ME" "CV" "PO" {
			foreach mv in "2" "4" "6" "8" {		
				di in green _col(2) "`y'" _col(10) "`x'" _col(14) ".`mv'" in yellow /*
				*/ _col(`cl1') %4.3f vX_`x'`i'4_`mv' _col(`cl2') %4.3f vX_`x'`i'2_`mv' _col(`cl3') %4.3f vX_`x'`i'1_`mv' _col(`cl4') %4.3f vX_`x'`i'3_`mv' /*
				*/ _col(`cl5') %4.3f vX_`x'`i'8_`mv' _col(`cl6') %4.3f vX_`x'`i'6_`mv' _col(`cl7') %4.3f vX_`x'`i'7_`mv' _col(`cl8') %4.3f vX_`x'`i'5_`mv'
			}
		}
	}

	di _newline(2) "Interaction table"
	di _col(2) "analysis A: complete case analysis"
	di _col(2) "analysis B: mi with outcome NOT included in model (and not predicted)"
	di _col(2) "analysis C: mi with outcome included in model but NOT predicted"
	di _col(2) "analysis D: mi with outcome included in model AND predicted"
	di _col(2) "analysis E: mi with outcome included in model AND predicted but then deleted"
	di _col(2) "analysis F: as in C but also including outcome2 in mi model"
	di _col(2) "analysis G: as in D but also including outcome2 in mi model"
	di _col(2) "analysis H: as in D but without the covariate"	
	//cols
    di in green _col(`cl1') "A" _col(`cl2') "B" _col(`cl3') "C" _col(`cl4') "D" _col(`cl5') "E" _col(`cl6') "F" _col(`cl7') "G" _col(`cl8') "H"
	//line output
	local i=0
	foreach y in "MCAR" "MAR" "MNAR" {
		local i=`i'+1
		foreach x in "MB" "ME" "CV" "PO" {
			foreach mv in "2" "4" "6" "8" {		
				di in green _col(2) "`y'" _col(10) "`x'" _col(14) ".`mv'" in yellow /*
				*/ _col(`cl1') %4.3f vI_`x'`i'4_`mv' _col(`cl2') %4.3f vI_`x'`i'2_`mv' _col(`cl3') %4.3f vI_`x'`i'1_`mv' _col(`cl4') %4.3f vI_`x'`i'3_`mv' /*
				*/ _col(`cl5') %4.3f vI_`x'`i'8_`mv' _col(`cl6') %4.3f vI_`x'`i'6_`mv' _col(`cl7') %4.3f vI_`x'`i'7_`mv' _col(`cl8') %4.3f vI_`x'`i'5_`mv'
			}
		}
	}
	
	di _newline(2) "Exposure table (SE)"
	di _col(2) "analysis A: complete case analysis"
	di _col(2) "analysis B: mi with outcome NOT included in model (and not predicted)"
	di _col(2) "analysis C: mi with outcome included in model but NOT predicted"
	di _col(2) "analysis D: mi with outcome included in model AND predicted"
	di _col(2) "analysis E: mi with outcome included in model AND predicted but then deleted"
	di _col(2) "analysis F: as in C but also including outcome2 in mi model"
	di _col(2) "analysis G: as in D but also including outcome2 in mi model"
	di _col(2) "analysis H: as in D but without the covariate"
	//cols
    di in green _col(`cl1') "A" _col(`cl2') "B" _col(`cl3') "C" _col(`cl4') "D" _col(`cl5') "E" _col(`cl6') "F" _col(`cl7') "G" _col(`cl8') "H"
	//line output
	local i=0
	foreach y in "MCAR" "MAR" "MNAR" {
		local i=`i'+1
		foreach x in "MB" "ME" "CV" "PO" {
			foreach mv in "2" "4" "6" "8" {		
				di in green _col(2) "`y'" _col(10) "`x'" _col(14) ".`mv'" in yellow /*
				*/ _col(`cl1') %5.4f seE_`x'`i'4_`mv' _col(`cl2') %5.4f seE_`x'`i'2_`mv' _col(`cl3') %5.4f seE_`x'`i'1_`mv' _col(`cl4') %5.4f seE_`x'`i'3_`mv' /*
				*/ _col(`cl5') %5.4f seE_`x'`i'8_`mv' _col(`cl6') %5.4f seE_`x'`i'6_`mv' _col(`cl7') %5.4f seE_`x'`i'7_`mv' _col(`cl8') %5.4f seE_`x'`i'5_`mv'
			}
		}
	}	

	di _newline(2) "Covariate table (SE)"
	di _col(2) "analysis A: complete case analysis"
	di _col(2) "analysis B: mi with outcome NOT included in model (and not predicted)"
	di _col(2) "analysis C: mi with outcome included in model but NOT predicted"
	di _col(2) "analysis D: mi with outcome included in model AND predicted"
	di _col(2) "analysis E: mi with outcome included in model AND predicted but then deleted"
	di _col(2) "analysis F: as in C but also including outcome2 in mi model"
	di _col(2) "analysis G: as in D but also including outcome2 in mi model"
	di _col(2) "analysis H: as in D but without the covariate"
	//cols
    di in green _col(`cl1') "A" _col(`cl2') "B" _col(`cl3') "C" _col(`cl4') "D" _col(`cl5') "E" _col(`cl6') "F" _col(`cl7') "G" _col(`cl8') "H"
	//line output
	local i=0
	foreach y in "MCAR" "MAR" "MNAR" {
		local i=`i'+1
		foreach x in "MB" "ME" "CV" "PO" {
			foreach mv in "2" "4" "6" "8" {		
				di in green _col(2) "`y'" _col(10) "`x'" _col(14) ".`mv'" in yellow /*
				*/ _col(`cl1') %5.4f seX_`x'`i'4_`mv' _col(`cl2') %5.4f seX_`x'`i'2_`mv' _col(`cl3') %5.4f seX_`x'`i'1_`mv' _col(`cl4') %5.4f seX_`x'`i'3_`mv' /*
				*/ _col(`cl5') %5.4f seX_`x'`i'8_`mv' _col(`cl6') %5.4f seX_`x'`i'6_`mv' _col(`cl7') %5.4f seX_`x'`i'7_`mv' _col(`cl8') %5.4f seX_`x'`i'5_`mv'
			}
		}
	}

	di _newline(2) "Interaction table (SE)"
	di _col(2) "analysis A: complete case analysis"
	di _col(2) "analysis B: mi with outcome NOT included in model (and not predicted)"
	di _col(2) "analysis C: mi with outcome included in model but NOT predicted"
	di _col(2) "analysis D: mi with outcome included in model AND predicted"
	di _col(2) "analysis E: mi with outcome included in model AND predicted but then deleted"
	di _col(2) "analysis F: as in C but also including outcome2 in mi model"
	di _col(2) "analysis G: as in D but also including outcome2 in mi model"
	di _col(2) "analysis H: as in D but without the covariate"	
	//cols
    di in green _col(`cl1') "A" _col(`cl2') "B" _col(`cl3') "C" _col(`cl4') "D" _col(`cl5') "E" _col(`cl6') "F" _col(`cl7') "G" _col(`cl8') "H"
	//line output
	local i=0
	foreach y in "MCAR" "MAR" "MNAR" {
		local i=`i'+1
		foreach x in "MB" "ME" "CV" "PO" {
			foreach mv in "2" "4" "6" "8" {		
				di in green _col(2) "`y'" _col(10) "`x'" _col(14) ".`mv'" in yellow /*
				*/ _col(`cl1') %5.4f seI_`x'`i'4_`mv' _col(`cl2') %5.4f seI_`x'`i'2_`mv' _col(`cl3') %5.4f seI_`x'`i'1_`mv' _col(`cl4') %5.4f seI_`x'`i'3_`mv' /*
				*/ _col(`cl5') %5.4f seI_`x'`i'8_`mv' _col(`cl6') %5.4f seI_`x'`i'6_`mv' _col(`cl7') %5.4f seI_`x'`i'7_`mv' _col(`cl8') %5.4f seI_`x'`i'5_`mv'
			}
		}
	}	
	
	di _newline(2) "Successful convergence (counts)"
	di _col(2) "analysis A: complete case analysis"
	di _col(2) "analysis B: mi with outcome NOT included in model (and not predicted)"
	di _col(2) "analysis C: mi with outcome included in model but NOT predicted"
	di _col(2) "analysis D: mi with outcome included in model AND predicted"
	di _col(2) "analysis E: mi with outcome included in model AND predicted but then deleted"
	di _col(2) "analysis F: as in C but also including outcome2 in mi model"
	di _col(2) "analysis G: as in D but also including outcome2 in mi model"
	di _col(2) "analysis H: as in D but without the covariate"	
	//cols
    di in green _col(`cl1') "A" _col(`cl2') "B" _col(`cl3') "C" _col(`cl4') "D" _col(`cl5') "E" _col(`cl6') "F" _col(`cl7') "G" _col(`cl8') "H"
	//line output
	local i=0
	foreach y in "MCAR" "MAR" "MNAR" {
		local i=`i'+1
		foreach mv in "2" "4" "6" "8" {		
			di in green _col(2) "`y'" _col(10) ".`mv'" in yellow /*
			*/ _col(`cl1') %6.0f cnt_`i'4_`mv' _col(`cl2') %6.0f cnt_`i'2_`mv' _col(`cl3') %6.0f cnt_`i'1_`mv' _col(`cl4') %6.0f cnt_`i'3_`mv' /*
			*/ _col(`cl5') %6.0f cnt_`i'8_`mv' _col(`cl6') %6.0f cnt_`i'6_`mv' _col(`cl7') %6.0f cnt_`i'7_`mv' _col(`cl8') %6.0f cnt_`i'5_`mv'
		}
	}		
	
	qui log close _all
	
end
